import random

import pygame

from globals import *
from helper import *
import sprite
import screen
import spritesheet
import status
import map
from screen import scr
from sprite import XSprite
from sprite import XGroup

class BossFragment(XSprite):
	def __init__(self, sheet, size, parent):
		self.parent = parent
		XSprite.__init__(self, sheet, size)

	def draw(self, surface):
		#r1 = pygame.Rect(scr.screen_position((self.location[0] + self.parent.location[0], self.location[1] + self.parent.location[1])), (self.size[0] * TILEX, self.size[1] * TILEY))
		#r2 = self.hitbox.move(r1.left, r1.top)
		#pygame.draw.rect(surface, (200, 200, 200), r2, 1)
		#r1 = self.get_rectangle()
		#r2 = self.hitbox.move(r1.left, r1.top)
		#t = scr.screen_position((r2.left, r2.top))
		#r3 = pygame.Rect(t[0], t[1], r2.width, r2.height)
		#pygame.draw.rect(surface, (200, 200, 200), r3, 1)
		
		surface.blit(self.sheet.surf, scr.screen_position((self.location[0] + self.parent.location[0], self.location[1] + self.parent.location[1])), self.sheet.frame(self.size, self.frame))
	
	def upv(self):
		self.location = (self.location[0] + self.vx, self.location[1] + self.vy)

	def get_rectangle(self):
		map_px = scr.map.size[0] * TILEX
		map_py = scr.map.size[1] * TILEY
		px = (self.location[0] + self.parent.location[0]) % map_px
		py = (self.location[1] + self.parent.location[1]) % map_py
		if ((self.location[0] + self.parent.location[0]) - (scr.mapx * TILEX + scr.offx)) %  map_px > (map_px / 2):
			px -= map_px
		#py = self.location[1]
		if ((self.location[1] + self.parent.location[1]) - (scr.mapy * TILEY + scr.offy)) %  map_py > (map_py / 2):
			py -= map_py
		return pygame.Rect((px, py), (self.size[0] * TILEX, self.size[1] * TILEY))

	def collides_with(self, other):
		r1 = self.get_rectangle()
		r2 = other.get_rectangle()
		return self.hitbox.move(r1.left, r1.top).colliderect(other.hitbox.move(r2.left, r2.top)) 

class BossCannonShot(sprite.Shot):
	def __init__(self, delta):
		sprite.Shot.__init__(self, delta, 'S01', (2,1))
		self.hitbox = pygame.Rect(4, 4, 24, 8)
		self.damage = 2
		self.hostile = True
		self.frame = 3
	
	def hit_target(self, target):
		target.hit(self)
		# don't die!


class Stachel(sprite.Enemy):
	def __init__(self):
		sprite.Enemy.__init__(self,'E01', (8,3))
		self.body = BossFragment('E01', (4,2), self)
		self.rarm = BossFragment('E01', (1,1), self)
		self.larm = BossFragment('E01', (1,1), self)
		self.cannon = BossFragment('E01', (2,1), self)		
		self.claw = BossFragment('E01', (2,1), self)
		self.leg1 = BossFragment('E01', (1,2), self)
		self.leg2 = BossFragment('E01', (1,2), self)
		self.leg3 = BossFragment('E01', (1,2), self)
		self.bleg1 = BossFragment('E01', (1,2), self)
		self.bleg2 = BossFragment('E01', (1,2), self)
		self.bleg3 = BossFragment('E01', (1,2), self)
		
		self.attacks = [self.ac_shoot, self.ac_sting, self.ac_charge, self.ac_jump]
		
		for l in [self.leg1, self.leg2, self.leg3, self.bleg1, self.bleg2, self.bleg3]:
			l.hitbox = pygame.Rect(0, 0, 0, 0)
		
		self.cycle = 0
		
		#self.parts = [self.bleg1, self.bleg2, self.bleg3, self.larm, self.claw, self.body, self.rarm, self.cannon, self.leg3, self.leg2, self.leg1]
		self.parts = [self.bleg1, self.bleg2, self.bleg3, self.claw, self.larm, self.body, self.rarm, self.cannon, self.leg3, self.leg2, self.leg1]
		self.damage = 0
		self.hp = 92
		self.framecount = 0
		
		self.body.frame = 21
		self.rarm.frame = 242
		self.larm.frame = 262
		self.cannon.frame = 120
		self.claw.frame = 130
		
		self.walk_form()
	
	def draw(self, surface):
		for p in self.parts:
			p.draw(surface)
	
	def walk_form(self):
		self.body.location = (16, 7)
		self.claw.location = (4, 32)
		self.larm.location = (30,28)
		self.cannon.location = (18, 28)
		self.rarm.location = (34, 24)
		self.leg1.location = (50, 16)
		self.leg2.location = (58, 16)
		self.leg3.location = (66, 16)		
		self.bleg1.location = (34, 16)
		self.bleg2.location = (42, 16)
		self.bleg3.location = (50, 16)
		self.leg1.frame = 111
		self.leg2.frame = 112
		self.leg3.frame = 113
		self.bleg1.frame = 108
		self.bleg2.frame = 109
		self.bleg3.frame = 110
		self.leg1.down = True
		self.leg2.down = True
		self.leg3.down = True
		self.bleg1.down = True
		self.bleg2.down = True
		self.bleg3.down = True
	
	def crouch_form(self):
		self.body.location = (16, 16)
		self.claw.location = (0, 32)
		self.larm.location = (32, 32)
		self.cannon.location = (24, 32)
		self.rarm.location = (36, 32)
		self.leg1.location = (50, 16)
		self.leg2.location = (58, 16)
		self.leg3.location = (66, 16)		
		self.bleg1.location = (34, 16)
		self.bleg2.location = (42, 16)
		self.bleg3.location = (50, 16)
		self.leg1.frame = 105
		self.leg2.frame = 106
		self.leg3.frame = 107
		self.bleg1.frame = 102
		self.bleg2.frame = 103
		self.bleg3.frame = 104
	
	def raise_leg(self, leg):
		if not leg.down:
			return
		leg.frame -= 6
		leg.location = (leg.location[0], leg.location[1] - 8)
		leg.down = False
	
	def lower_leg(self, leg):
		if leg.down:
			return
		leg.frame +=6
		leg.location = (leg.location[0], leg.location[1] + 8)
		leg.down = True

	def raise_all(self):
		for l in [self.leg1, self.leg2, self.leg3, self.bleg1, self.bleg2, self.bleg3]:
			self.raise_leg(l)
	
	def lower_all(self):
		for l in [self.leg1, self.leg2, self.leg3, self.bleg1, self.bleg2, self.bleg3]:
			self.lower_leg(l)

	def ac_start(self, plr):
		seek = self.seek_to(plr)
		if seek[1] > -100:
			self.framecount += 1
		
		if self.framecount < 10:
			self.crouch_form()
		else:	
			self.framecount  = 0
			self.action = self.attacks[random.randint(0, len(self.attacks)) - 1]
	
	#launch bolts with cannon
	def ac_shoot(self, plr):
		if self.cannon not in self.parts:
			self.action = self.ac_start
			return
		
		self.framecount += 1
		self.cannon.upv()
		self.rarm.upv()
		
		if self.framecount == 1:
			self.rarm.vx = -1
			self.cannon.vx = -4
		elif self.framecount == 4:
			self.rarm.vx = 0
			self.cannon.vx = 0
		elif self.framecount == 16 or self.framecount == 32 or self.framecount == 48:
			shot = BossCannonShot((-4, 0))
			shot.location = sum_tuple(self.location, (-8, 32))
			self.children.add(shot)
		elif self.framecount == 52:
			self.rarm.vx = 1
			self.cannon.vx = 4
		elif self.framecount == 55:
			self.rarm.vx = 0
			self.cannon.vx = 0
		elif self.framecount == 60:
			self.framecount = 0
			self.action = self.ac_start

	
	# leap forward, then leap back
	def ac_jump(self, plr):
		self.framecount += 1
		
		if self.framecount < 10:
			self.crouch_form()
		elif self.framecount == 11:
			self.walk_form()
		elif self.framecount == 13:
			self.vx = -11
			self.vy = -10
			self.ay = 1
		
		if self.framecount > 15 and not self.can_fall():
			self.vx = 0
			self.vy = 0
			self.ay = 0
			self.framecount = 0
			self.action = self.ac_jump_back
		
	
	def ac_jump_back(self, plr):
		self.framecount += 1
		
		if self.framecount < 10:
			self.crouch_form()
		elif self.framecount == 11:
			self.walk_form()
		elif self.framecount == 13:
			self.vx = 11
			self.vy = -10
			self.ay = 1
		
		if self.framecount > 15 and not self.can_fall():
			self.vx = 0
			self.vy = 0
			self.ay = 0
			self.framecount = 0
			self.action = self.ac_start

	# run forward, lunge with claw, walk back
	def ac_charge(self, plr):
		if self.claw not in self.parts:
			self.action = self.ac_start
			return
		
		if self.framecount == 0:
			self.vx = -4
			self.walk_form()
		
		self.framecount += 1
		
		if not self.obstructed((-20, 0)):
			legframe = (self.framecount / 2) % 6
			if legframe == 0:
				self.raise_leg(self.leg1)
				self.lower_leg(self.bleg3)
			if legframe == 1:
				self.lower_leg(self.leg3)
				self.raise_leg(self.bleg2)
			if legframe == 2:
				self.raise_leg(self.leg2)
				self.lower_leg(self.bleg1)
			if legframe == 3:
				self.lower_leg(self.leg1)			
				self.raise_leg(self.bleg3)
			if legframe == 4:
				self.raise_leg(self.leg3)
				self.lower_leg(self.bleg2)
			if legframe == 5:
				self.lower_leg(self.leg2)
				self.raise_leg(self.bleg1)
		else:
			self.lower_all()
			self.vx = 0
			self.framecount = 0
			self.action = self.ac_claw_jab
	
	def ac_claw_jab(self, plr):
		self.framecount += 1
		self.claw.upv()
		self.larm.upv()
		
		if self.framecount == 4:
			self.claw.vy = -4
			self.claw.vx = 3
			self.larm.vy = -3
			self.larm.vx = 3
		if self.framecount == 8:
			self.claw.vy = 0
			self.claw.vx = 0
			self.larm.vy = 0
			self.larm.vx = 0
			self.larm.position = (32, 32)
		if self.framecount == 10:
			self.claw.vx = -8
			self.larm.vx = -8
		if self.framecount == 14:
			self.claw.vx = 8
			self.larm.vx = 8
		if self.framecount == 18:
			self.claw.vy = 0
			self.claw.vx = 0
			self.larm.vy = 0
			self.larm.vx = 0
		if self.framecount == 36:
			self.claw.vx = -8
			self.larm.vx = -8
		if self.framecount == 40:
			self.claw.vx = 8
			self.larm.vx = 8
		if self.framecount == 44:
			self.claw.vy = 0
			self.claw.vx = 0
			self.larm.vy = 0
			self.larm.vx = 0			
			self.framecount = 0
			self.action = self.ac_jump_back
	
	# raise tail, shoot spray
	def ac_sting(self, plr):
		self.action = self.ac_start


	def hit(self, thing):
		self.hp -= thing.damage
		if self.hp <= 0:
			self.action = self.ac_die
		else:
			print self.hp

	def ac_die(self, plr):
		XSprite.ac_die(self)

	def collides_with(self, other):
		for p in self.parts:
			if p.collides_with(other):
				return True
		return False


