import os

import pygame
from pygame.locals import *

from globals import *
import screen
import status
import font

class Menu:
	def __init__(self):
		self.get_bg()
		self.items = []
		self.selected = 0
		self.cancelable = True
	
	def get_bg(self):
		self.bg = pygame.display.get_surface().copy()
		shade = pygame.surface.Surface((RESX, RESY), 0, self.bg)
		shade.fill((64, 64, 64))
		shade.set_alpha(128)
		self.bg.blit(shade, (0,0))	
	
	def go(self):
		menu_done = False
		
		pygame.display.get_surface().blit(self.bg, (0, 0))
		selected = 0
		clock = pygame.time.Clock()
		
		font.write(font.font, "Parts supply: " + str(status.parts), (255, 255, 255), (170, 200))
		font.write(font.font, "Press jump key to resume play", (255, 255, 255), (0, 200))
		
		while not menu_done:
			for event in pygame.event.get():
				if event.type == KEYDOWN:
					if event.key == Key.jump and self.cancelable:
						menu_done = True
					elif event.key == Key.up:
						selected = (selected - 1) % len(self.items)
					elif event.key == Key.down:
						selected = (selected + 1) % len(self.items)
					elif event.key == Key.select:
						self.item_selected(selected)
			
			for item in self.items:
				item.write(False)
			
			self.items[selected].write(True)
			
			pygame.display.flip()
			
			clock.tick(frame_rate)

class MenuItem:
	def __init__(self, text, position):
		self.text = text
		self.position = position
		self.font = font.font
		self.disabled = False
	
	def write(self, active):
		if active:
			if not self.disabled:
				color = (160, 160, 255)
			else:
				color = (140, 140, 140)
		else:
			if not self.disabled:
				color = (0, 0, 190)
			else:
				color = (70, 70, 70)
		
		# this is to work around a silly bug in SDL or freetype
		try:
			pygame.display.get_surface().blit(self.font.render(self.text, False, color), self.position)
		except:
			pygame.display.get_surface().blit(self.font.render(self.text, True, color), self.position)

menu_stuff = [
	("Increase maximum life: ", status.life_up_cost, status.maxlife_up, 0),
	("Increase life regeneration: ", status.life_regen_up_cost, status.liferegen_up, 1),
	("Increase maximum ammo: ", status.ammo_up_cost, status.maxammo_up, 2),
	("Increase ammo regeneration: ", status.ammo_regen_up_cost, status.ammoregen_up, 3),
	("Increase firing speed: ", status.shot_speed_up_cost, status.firespeed_up, 4),
	("Increase firing speed: ", status.shot_speed_up_cost, status.firespeed_up, 5),
	("Use basic gun: ", status.normal_cost, status.use_normal, 7),
	("Use rapid fire gun: ", status.rapid_cost, status.use_rapid, 8),
	("Use beam gun: ", status.beam_cost, status.use_beam, 9),
	("Use spread gun: ", status.spread_cost, status.use_spread, 10),
	("Use flamethrower gun: ", status.flame_cost, status.use_flame, 11),
	
	("Dash upgrade: ", status.dash_cost, status.buy_dash, 13),
	("Dodge roll upgrade: ", status.roll_cost, status.buy_roll, 14),
	("High jump upgrade: ", status.hijump_cost, status.buy_hijump, 15)
	]

class TownMenu(Menu):
	def __init__(self, town):
		Menu.__init__(self)
		self.build_menu()
	
	def build_menu(self):
		self.items = []
		for x in menu_stuff:
			item = MenuItem(x[0] + str(x[1]()) + " parts", (50, (x[3] + 3) * 10))
			if x[1]() == 0 or x[1]() > status.parts:
				item.disabled = True
			self.items.append(item)
	
	def item_selected(self, num):
		if self.items[num].disabled:
			return
		
		status.parts -= menu_stuff[num][1]()
		menu_stuff[num][2]()
		
		self.build_menu()
		
	
	def done(self):
		self.menu_done = True






